<?php
require_once 'dto/Card.php';
require_once 'CardDao.php';

class AddressBookService {

    const DSN = 'sqlite:/temp/flex_address_book.sq3';

    private $cardDao;

    /**
     * コンストラクタ。
     * CardDaoオブジェクトを生成します。
     * SQLiteデータベースにcardテーブルが存在しない場合、
     * 新規に作成します。
     */
    public function __construct() {
        $this->cardDao = new CardDao(self::DSN);
        $this->cardDao->createTableIfNotExists();
    }

    /**
     * 保存されている全Cardオブジェクトを名前順の配列で返します。
     */
    public function getCards() {
        return $this->cardDao->findAll();
    }

    /**
     * Flex2から送信されたCardオブジェクトを保存します。
     */
    public function saveCard(Card $card) {
        $card->portraitUrl = $this->saveByteArrayAsImageFile($card);
        if (empty($card->id)) {
            // 新規保存
            return $this->cardDao->insert($card);
        } else {
            // 更新保存
            return $this->cardDao->update($card);
        }
    }

    /**
     * 指定されたIDのレコードを削除します。
     */
    public function deleteCard($cardId) {
        $card = $this->cardDao->findById($cardId);
        if ($card) {
            $this->deletePortraitFileIfExists($card);
            $this->cardDao->delete($card);
        }
    }

    /**
     * 指定されたCardオブジェクトの似顔絵画像ファイルを削除します。
     */
    private function deletePortraitFileIfExists(Card $card) {
        if (!empty($card->portraitUrl)) {
            $filepath = $this->getPortraitFilepath($card->portraitUrl);
            if (file_exists($filepath)) {
                unlink($filepath);
            }
        }
    }

    /**
     * 指定されたCardオブジェクトに似顔絵データがあれば
     * ファイルに保存し、画像URLを返します。
     */
    private function saveByteArrayAsImageFile(Card $card) {
        $data = $card->portrait;
        if ($data != null) {
            $url = empty($card->portraitUrl) ?
                $this->generatePortraitUrl() : $card->portraitUrl;
            $filepath = $this->getPortraitFilepath($url);
            $fp = fopen($filepath, 'wb');
            $length = count($data);
            for ($i = 0; $i < $length; $i++) {
                fwrite($fp, chr($data[$i]));
            }
            fclose($fp);
            return $url;
        }
        return null;
    }

    /**
     * 似顔絵画像ファイルのURLを生成します。
     */
    private function generatePortraitUrl() {
        return '/portraits/' . time() . '.png';    
    }

    /**
     * 似顔絵画像ファイルのファイルパスを返します。
     */
    private function getPortraitFilepath($url) {
        $docRoot = dirname(dirname(dirname(dirname(__FILE__))));
        $path = $docRoot . $url;
        if (!file_exists(dirname($path))) {
            mkdir(dirname($path));
        }
        return $path;
    }
}

?>

