<?php
/*******************************************************************
 * SecurityConfigHandler.php
 * Copyright (C) 2006 Midnight Coders, LLC
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
 * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
 * NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE
 * LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION
 * OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION
 * WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 *
 * The software is licensed under the GNU General Public License (GPL)
 * For details, see http://www.gnu.org/licenses/gpl.txt.
 ********************************************************************/
 
 require_once(WebOrb . "Config/ORBConfigHandler.php");
 require_once(WebOrb . "Util/Invocation.php");
 require_once(WebOrb . "Util/ObjectFactories.php"); 
 require_once(WebOrb . "Security/AccessConstraint.php");  
 
 class SecurityConfigHandler
    extends ORBConfigHandler
 {
   
 		private $m_securityNode;
        private $m_deploymentNode;
        private $m_authenticationHandler;
        private $m_authorizationHandler;
		private $m_accessConstraintsNode;
		private $m_secureResourcesNode;
		private $m_RESTRICTION_TYPES;

    public function __construct()
    {
		$this->m_RESTRICTION_TYPES = array();
		
		$this->m_RESTRICTION_TYPES["SingleIPRestriction"] = "singleIP";
		$this->m_RESTRICTION_TYPES["IPRangeRestriction"] = "ipRange";
		$this->m_RESTRICTION_TYPES["HostNameRestriction"] = "hostName";
		$this->m_RESTRICTION_TYPES["RoleNameRestriction"] = "roleName";
    }

    public function configure($parent, $configContext, DOMNode $section) 
    {
      $section = $section->getElementsByTagName("security")->item(0);
      
      $this->setORBConfig($configContext);
      $this->m_securityNode = $section;
      $this->m_deploymentNode = $section->getElementsByTagName("deploymentMode")->item(0);
      
      $security = $configContext->getSecurity();
      $security->setDeploymentMode($this->m_deploymentNode->textContent);
      
      $this->m_authenticationHandler = $section->getElementsByTagName("authenticationHandler")->item(0);
      $this->m_authorizationHandler = $section->getElementsByTagName("authorizationHandler")->item(0);
      
      if($this->m_authorizationHandler != null && strlen($this->m_authorizationHandler->textContent) != 0)
      {
          $authHandler = ObjectFactories::createServiceObject($this->m_authorizationHandler->textContent);
          $security->setAuthorizationHandler($authHandler);
      }
      
      $this->m_accessConstraintsNode = $section->getElementsByTagName("access-constraints")->item(0);
      $this->m_secureResourcesNode = $section->getElementsByTagName("secure-resources")->item(0);   
      
      $this->processConstraints($this->m_accessConstraintsNode);
      $this->processResources($this->m_secureResourcesNode);     
      
      return $this;  
    }
    
    public function getAuthenticationHandler()
    {
        return $this->m_authenticationHandler->textContent;
    }
    
    public function addAccessConstraint( AccessConstraint $constraint )
    {
      $accessConstraintElement = $this->getAccessConstraintElement( $constraint->name );
      
      if($accessConstraintElement == null)
      {
        $doc = $this->m_securityNode->ownerDocument;
        $accessConstraintElement = $doc->createElement("access-constraint");
        $accessConstraintElement->setAttribute("action",$constraint->getAction());
        $nameElement = $doc->createElement("name");
        $nameElement->textContent = $constraint->name;
        
        $accessConstraintElement->appendChild($nameElement);
        $this->accessConstraintsNode->appendChild($accessConstraintElement);
      }
      
      
      $restrictions = $constraint->getRestrictions();
      
      for($i = 0; $i < count($restrictions); $i++)
      {
        $restriction = $restrictions[$i];
        
        $this->addRestriction($accessConstraintElement,$restriction);
      }
    }
    
	public function removeAccessConstraint( $constraintName )
	{
		$accessConstraintsElement = $this->getAccessConstraintElement( $constraintName );

		if( $accessConstraintsElement != null )
			$accessConstraintsNode->removeChild( $accessConstraintsElement );
	}
	
	public function addResourceConstraints($resourceName, $constraintNames)
	{
      $secureResource = $this->getSecureResourceElement($resourceName);
      $doc = $this->m_securityNode->ownerDocument;
      
      if($secureResource == null)
      {
        $secureResource = $doc->createElement("secure-resource");
        $resource = $doc->createElement("resource");
        $resource->textContent = $resourceName;
        
        $secureResource->appendChild($resource);
        $this->m_secureResourcesNode->appendChild($secureResource);
      }
      
      for($i = 0; $i < count($constraintNames); $i++)
      {
        $constraint = $doc->createElement("constraint-name");
        $constraint->textContent = $constraintNames[$i];
        $secureResource->appendChild($constraint);
      }
    }
	
	public function removeResourceConstraints($resourceName, $constraintNames)
	{
		$secureResource = $this->getSecureResourceElement( $resourceName );
		$constraints = $secureResource->getElementsByTagName( "constraint-name" );
		$constraintsToRemove = $constraintNames;
		$elementsToDetach = array();        
		
		foreach($constraints as $constraint)
		{				
		  $constraintName = trim($constraint->textContent);
				
		  if(in_array($constraintName,$constraintsToRemove))
             $elementsToDetach[] = $constraint;
		}	
            
        
        foreach($elementsToDetach as $constraint)
            $secureResource->removeChild($constraint);
            
        if(!$secureResource->hasChildNodes())
            $this->m_secureResourcesNode->removeChild($secureResource);
		
    }
    
	public function addRestriction($constraintName,$action, IRestriction $restriction)
	{
      $accessConstraintElement = $this->getAccessConstraintElement($constraintName);
      
      if($accessConstraintElement == null)
      {
        $accessConstraint = new AccessConstraint($constraintName,$action);
        $accessConstraint->addRestriction($restriction);
        $this->addAccessConstraint($accessConstraint);
      }
      else
      {
        $this->addRestriction( $accessConstraintElement, $restriction );  
      }
    }
	
	public function removeRestriction($constraintName, IRestriction $restriction)
	{
      $accessConstraintElement = $this->getAccessConstraintElement( $constraintName );
      
	  if( $accessConstraintElement == null )
		  throw new ArgumentException( "unknown constraint name - " + $constraintName );      
      
	  $this->removeRestriction2( $accessConstraintElement, $restriction );      
	  
	  if(!$accessConstraintElement->hasChildNodes())
	   $this->accessConstraintsNode->removeChild($accessConstraintElement);
    }

    private function addRestriction2($accessConstraintElement,IRestriction $restriction)
    {
		$element = $this->getRestrictionElement( $accessConstraintElement, $restriction );
        
        if($element != null && $element->parentNode !== null)      
            $accessConstraintElement->appendChild($element);
    }
    
    private function removeRestriction2($accessConstraintElement, IRestriction $restriction)
    {
      $element = $this->getRestrictionElement( $accessConstraintElement, $restriction );
      
	  if( $element != null && $element->parentNode != null )
			$accessConstraintElement->removeChild( $element );      
    }
    
    private function getRestrictionElement($accessConstraintElement,IRestriction $restriction)
    {
      $restrictionClassName = get_class($restriction);
      $reflectionClass = new ReflectionClass($restrictionClassName);
      $methodName = $this->m_RESTRICTION_TYPES[$restrictionClassName];
      $methodInfo = $reflectionClass->getMethod($methodName);
      
      return Invocation::invoke($this,$methodInfo,array($accessConstraintElement,$restriction));
    }
    
    private function getAccessConstraintElement($constraintName)
    {
      foreach($this->m_accessConstraintsNode->childNodes as $node)
      {
        if(!($node instanceof DOMElement))
            continue;
            
        if(strtolower($node->getElementsByTagName("name")->item(0)->textContent) 
            == strtolower($constraintName))
            return $node;
      }
      
      return null;
    }
    
    private function getSecureResourceElement($resourceName)
    {
      foreach($this->m_secureResourcesNode as $node)
      {
         if(!($node instanceof DOMElement))
            continue;
            
        if(strtolower($node->getElementsByTagName("resource")->item(0)->textContent) 
            == strtolower($resourceName))
            return $node;                     
      }
      
      return null;
    }
    
    private function processResources($resources)
    {
      foreach($resources->childNodes as $node) 
      {
         if(!($node instanceof DOMElement))
            continue;
            
         $resourceId = $this->getElementText($node,"resource");
         $constraints = $node->getElementsByTagName( "constraint-name" );
         $constraintNames = array();
         
         
         
         for($i = 0; $i < $constraints->length; $i++)
            $constraintNames[$i] = trim($constraints->item($i)->textContent);
            
         $resourceAuthHandlers = $node->getElementsByTagName("resourceAuthorizationHandler");
         $resourceAuthHandler = null;
         
         if($resourceAuthHandlers->length > 0)
         {
           $authHandlerClassName = $resourceAuthHandlers[0]->textContent;   
           $resourceAuthHandler = ObjectFactories::createServiceObject($authHandlerClassName);
         }
         
         $this->getORBConfig()->getSecurity()->secureResource($resourceId, 
                                                                $constraintNames, 
                                                                $resourceAuthHandler);
      }
    }
    
	private function processConstraints( $constraints )  
    {
	   $accessConstraints = array();      
	   
	   foreach($constraints->childNodes as $node)
	   {
		  if( !($node instanceof DOMElement) )
			continue;  
            
 		  $accessConstrObj = $this->processConstraint( $node );    
          $constraintName = $this->getElementText($node,"name");
          
          if($accessConstrObj != null)
            $accessConstraints[$constraintName] = $accessConstrObj;
       }
       
       $this->getORBConfig()->getSecurity()->setAccessConstraints($accessConstraints);
    }  
    
    private function processConstraint($contraintElement)
    {
         Log::log(LoggingConstants::SECURITY,"Called");
                
		$accessConstraintName = $this->getElementText( $contraintElement, "name" );
		$action = $contraintElement->getAttribute( "action" );  
        
        if($accessConstraintName == null)    
        {
          Log::log(LoggingConstants::ERROR,"missing 'name' element in access-constraint element");
		  return null;          
        }
        
		if($action == null)
		{
		  Log::log( LoggingConstants::ERROR, "missing 'action' attribute in the " . $accessConstraintName . " access-constraint element" );
		  return null;
		} 
        
        $accessConstraint = new AccessConstraint( $accessConstraintName, $action );
        
        $ipElements = $contraintElement->getElementsByTagName("IP");
        $this->processIPs( $ipElements, $accessConstraint );
        
        $ipRangeElements = $contraintElement->getElementsByTagName("IPrange");
        $this->processIPRanges( $ipRangeElements, $accessConstraint );
        
 		$hostNameElements = $contraintElement->getElementsByTagName( "hostname" );
		$this->processHostNames( $hostNameElements, $accessConstraint ); 
        
		$roleElements = $contraintElement->getElementsByTagName( "role" );
		$this->processRoles( $roleElements, $accessConstraint );       
                
        return $accessConstraint;
    }

    
 		
    private function processIPs( $ipNodes, AccessConstraint $accessConstraint )
	{
	  foreach($ipNodes as $node)
	  {
        $accessConstraint->addIP( trim($node->textContent) );
	  }
	}
	
	private function processIPRanges($ipRangeNodes, AccessConstraint $accessConstraint )
	{
	  foreach($ipRangeNodes as $node )
	  {
		  if( !($node instanceof DOMElement) )
			continue; 

		  $subnetAddres = $this->getElementText($node, "subnet-address" );
		  $subnetMask = $this->getElementText($node, "subnet-mask" );
		  
          $accessConstraint->addIPRange( $subnetAddres, $subnetMask );
	  }
	}
    
	
    private function processHostNames( $ipNodes, AccessConstraint $accessConstraint )
	{
	  foreach($ipNodes as $node)
	  {
		 if(!($node instanceof DOMElement) )
			continue; 

				
         $accessConstraint->addHostName(trim( $node->textContent));
	  }
	}	
	
	private function processRoles( $ipNodes, AccessConstraint $accessConstraint )
	{
	  foreach($ipNodes as $node)
	  {
		 if(!($node instanceof DOMElement) )
			continue;

         $roleName = trim($node->textContent);

         if( !$this->getORBConfig()->getSecurity()->checkRole( $roleName ) )
                Log::log( LoggingConstants::INFO, "Unknown role name in access constant " . $accessConstraint->getName() . ". Role name is " .  $roleName . ". Make sure the role name is listed in acl.xml" );

		$accessConstraint->addRole( $roleName );
	  }
	}
    
	private function getElementText( $element, $elementName )
	{
	  return trim($element->getElementsByTagName( $elementName )->item(0)->textContent);
	}	
	
	
	private function singleIP( $accessConstraintElement, $restriction )
	{
	  $ipAddress = $restriction->getIPAddress();
	  
	  $ipAddresses = $accessConstraintElement->getElementsByTagName( "IP" );  
      
 	  foreach($ipAddresses as $ipAddressElement)
	  {
		 if(!($ipAddressElement instanceof DOMElement) )
			continue;
				
         if( trim($ipAddressElement->textContent) == $ipAddress)
			return $ipAddressElement;
	  }
     
	  $addressElement = $accessConstraintElement->ownerDocument->createElement( "IP" );
	  $addressElement->textContent = $ipAddress;
			
      return $addressElement;              
    }
    
	private function ipRange( $accessConstraintElement, $restriction )
	{
	  $subnetAddress = $restriction->getSubnetAddress();
      $subnetMask = $restriction->getSubnetMask();
	  $ipRanges = $accessConstraintElement->getElementsByTagName( "IPrange" );
   
	  foreach($ipRanges as $ipRangeElement)
	  {
		 if(!($ipRangeElement instanceof DOMElement) )
			continue;

		 if($this->getElementText($ipRangeElement,"subnet-address") == $subnetAddress &&
		   $this->getElementText($ipRangeElement,"subnet-mask") == $subnetMask)
		 {
			return $ipRangeElement;
	     }
	  }
	  
	  $rangeElement = $accessConstraintElement->ownerDocument->createElement( "IPrange" );
	  $subnetAddressElement = $accessConstraintElement->ownerDocument->createElement( "subnet-address" );
	  $subnetMaskElement = $accessConstraintElement->ownerDocument->createElement( "subnet-mask" );
	  
	  $subnetAddressElement->textContent = $subnetAddress;
	  
	  $subnetMaskElement->textContent = $subnetMask;
			
      $rangeElement->appendChild( $subnetAddressElement );
	  $rangeElement->appendChild( $subnetMaskElement );
			
	  return $rangeElement;	  
	  
	 }
	 
	 private function hostName( $accessConstraintElement, $restriction )
	 {
        $hostName = $restriction->getHostName();
        $hostNames = $accessConstraintElement->getElementsByTagName( "hostname" );
        
        foreach($hostNames as $hostNameElement)  
        {
		 if(!($hostNameElement instanceof DOMElement) )
			continue;    
            
		 if(trim($hostNameElement->textContent) == $hostName)
			 return $hostNameElement;                  
        }
        
        $hNameElement = $accessConstraintElement->ownerDocument->createElement( "hostname" );
        $hNameElement->textContent = $hostName;
        
        return $hNameElement;
     }
	 
	 private function roleName( $accessConstraintElement, $restriction )
	 {
		$roleName = $restriction->getRoleName();
		$roleNames = $accessConstraintElement->getElementsByTagName( "role" );
            
		foreach($roleNames as $roleNameElement)
		{
		  if( !($roleNameElement instanceof DOMElement) )
			continue;

		  if(trim($roleNameElement->textContent) == $roleName)
			return $roleNameElement;
		}            
		
		$rNameElement = $accessConstraintElement->ownerDocument->createElement( "role" );
		$rNameElement->textContent = $roleName;
     }  	 
	 
 }

?>